`timescale 1ns/1ns

module eth_send_test(
	rst_n,
	
	sink_datavalid,
	sink_data,
	sink_dataclk,
	sink_sop,
	sink_eop,
	
	aclr,
	frame_bytes_len,
	wrusedw,
	
	//MII 接口信号	
	mii_tx_clk,
	mii_tx_en,
	mii_tx_er,
	mii_tx_data,
	
	mii_rx_clk,
	mii_rx_dv,
	mii_rx_er,
	mii_rx_data,
	
	phy_rst_n
);

	input rst_n;
	
	input	  aclr;
	input  [11:0]frame_bytes_len;
	output	[10:0]wrusedw;
	
	input sink_datavalid;
	input sink_dataclk;
	input [15:0]sink_data;
	input sink_sop;
	input sink_eop;	
	
	//MII 接口信号	
	input mii_tx_clk;	//MII接口发送时钟，由PHY芯片产生，25MHz
	output mii_tx_en;	//MII接口发送数据使能信号，高电平有效
	output mii_tx_er;	//发送错误，用以破坏数据包发送
	output [3:0]mii_tx_data; //MII接口发送数据线，FPGA通过该数据线将需要发送的数据依次送给PHY芯片
	output phy_rst_n;	// PHY 复位信号

	input mii_rx_clk;	//MII接口接收时钟，由PHY芯片产生，25MHz
	input mii_rx_dv;	//MII接口接收数据有效信号，高电平有效
	input mii_rx_er;	//接收错误，本实例中暂时忽略该信号
	input [3:0]mii_rx_data;	//MII接口数据总线，FPGA通过该数据线读取PHY芯片接收到的以太网数据
	
	assign phy_rst_n = 1;
	
	reg tx_go;
	wire send_done;
	wire [15:0]ip_total_len;	//IP数据包总长度
	wire [15:0]ip_checksum;
	wire [31:0]src_ip;
	wire [31:0]dst_ip;
	wire [15:0]udp_total_len;	//UDP数据包总长度
	wire [15:0]dst_port;	//目标端口号
	wire [15:0]src_port; //源端口号
	
	reg [15:0]data_total_len;//实际要发送的用户数据长度
	
	assign src_ip = 32'hc0_a8_00_02;
	assign dst_ip = 32'hc0_a8_00_03;

	assign ip_total_len = data_total_len + 8'd28;
	assign udp_total_len = data_total_len + 8'd8;
	
	assign dst_port = 16'd6000;	//目的端口号6000
	assign src_port = 16'd5000;	//源端口号5000

	wire fifo_rdreq;	//读发送缓存fifo请求
	reg [3:0] fifo_rddata; //发送缓存fifo数据输出
	wire fifo_rdclk;	//读发送缓存fifo时钟
	reg [11:0]data_cnt;
		
	wire [31:0]Crc_eth;
	wire CRC_EN;//CRC校验单元校验使能信号

	wire	  data_rdreq;
	wire	[3:0]  rddata;
	wire	[12:0]  rdusedw;
	reg frame_done;
	reg [11:0]state;
	reg frame_done_r;
	
	reg packet_state;
	
	localparam
		IDLE = 12'b0000_0000_0001,
		WRFIFO = 12'b0000_0000_0010,
		START_TX = 12'b0000_0000_0100,
		DELAY1 = 12'b0000_0000_1000,
		DELAY2 = 12'b0000_0001_0000,
		DELAY3 = 12'b0000_0010_0000,
		DELAY4 = 12'b0000_0100_0000,
		DELAY5 = 12'b0000_1000_0000,
		DELAY6 = 12'b0001_0000_0000,
		DELAY7 = 12'b0010_0000_0000,
		WAITDONE = 12'b0100_0000_0000,
		DELAY_BETWEEN_FRAME = 12'b1000_0000_0000;
		
	eth_send eth_send(
		.rst_n(rst_n),
		.tx_go(tx_go),
		.data_length(ip_total_len<<1),
//		.des_mac(48'h84_7b_eb_48_94_13),//目的网卡号
		.des_mac(48'hffffffffffff),//目的网卡号
		.src_mac(48'h00_0a_35_01_fe_c0),//源（FPGA板卡）网卡号
		.type_length(16'h08_00),	//协议类型
		.CRC_Result(Crc_eth),		//发送数据的CRC实时校验值
		.CRC_EN(CRC_EN),				//CRC校验单元校验使能信号
		.send_done(send_done),		//发送完成标志信号
		.fifo_rdreq(fifo_rdreq),	//读发送缓存fifo请求
		.fifo_rddata(fifo_rddata),	//发送缓存fifo数据输出
		.fifo_rdclk(fifo_rdclk),	//读发送缓存fifo时钟
		.mii_tx_clk(mii_tx_clk),	
		.mii_tx_en(mii_tx_en),		
		.mii_tx_er(mii_tx_er),
		.mii_tx_data(mii_tx_data)
	);

	crc32_d4 crc32_d4(
		.Clk(mii_tx_clk), 
		.Rst_n(rst_n), 
		.Data(mii_tx_data), 
		.Enable(CRC_EN), 
		.Initialize(~mii_tx_en), 
		.Crc(), 
		.CrcError(),
		.Crc_eth(Crc_eth)
	);
	
	checksum checksum(
		.ver(4'h4),
		.hdr_len(4'h5),
		.tos(8'h0),
		.total_len(ip_total_len),
		.id(16'h0),
		.offset(16'h0),
		.ttl(8'h40),
		.protocol(8'h11),
		.src_ip(src_ip),
		.dst_ip(dst_ip),
		.checksum_result(ip_checksum)
	);
	
	send_dcfifo send_dcfifo(
		.aclr(aclr),
		.data(sink_data),
		.rdclk(mii_tx_clk),
		.rdreq(data_rdreq),
		.wrclk(sink_dataclk),
		.wrreq(sink_datavalid & packet_state),
		.q(rddata),
		.rdusedw(rdusedw),
		.wrusedw(wrusedw)
	);
	
	always@(posedge sink_dataclk or negedge rst_n)
	if(!rst_n)
		packet_state <= 1'b0;
	else if(sink_sop)
		packet_state <= 1'b1;
	else if(sink_eop)
		packet_state <= 1'b0;
		
	reg [2:0]packet_state_r;
	
	always@(posedge mii_tx_clk)
		packet_state_r <= {packet_state_r[1:0],packet_state};
	
	//计数发送数据个数，用于产生对应的数据
	always@(posedge mii_tx_clk or negedge rst_n)
	if(!rst_n)
		data_cnt <= #1 12'd0;
	else if(fifo_rdreq)
		data_cnt <= #1 data_cnt + 1'b1;
	else
		data_cnt <= #1 12'd0;
		
	assign data_rdreq = data_cnt >= 56;
	
	//UDP包	
	always@(*)
	begin
		case(data_cnt)
			0   : fifo_rddata = 4'h5;//首部长度
			1   : fifo_rddata = 4'h4;//协议版本
			
			//服务类型
			2   : fifo_rddata = 4'h0;
			3   : fifo_rddata = 4'h0;
			
			//IP数据报总长度（IP报头+数据）
			4   : fifo_rddata = ip_total_len[11:8];
			5   : fifo_rddata = ip_total_len[15:12];
			6   : fifo_rddata = ip_total_len[3:0];
			7   : fifo_rddata = ip_total_len[7:4];
			
			//数据包标识
			8   : fifo_rddata = 4'h0;
			9   : fifo_rddata = 4'h0;
			10  : fifo_rddata = 4'h0;
			11  : fifo_rddata = 4'h0;
			
			//标识+分段偏移
			12  : fifo_rddata = 4'h0;
			13  : fifo_rddata = 4'h0;
			14  : fifo_rddata = 4'h0;
			15  : fifo_rddata = 4'h0;
			
			//生存时间
			16  : fifo_rddata = 4'h0;
			17  : fifo_rddata = 4'h4;
			
			//数据报类型 17： UDP
			18  : fifo_rddata = 4'h1;
			19  : fifo_rddata = 4'h1;
			
			//IP报头校验和 使用自动IP和校验逻辑生成的校验和值
			20  : fifo_rddata = ip_checksum[11:8];
			21  : fifo_rddata = ip_checksum[15:12];
			22  : fifo_rddata = ip_checksum[3:0];
			23  : fifo_rddata = ip_checksum[7:4];
			
			//源地址192.168.0.2
			24  : fifo_rddata = src_ip[27:24];
			25  : fifo_rddata = src_ip[31:28];
			26  : fifo_rddata = src_ip[19:16];
			27  : fifo_rddata = src_ip[23:20];
			28  : fifo_rddata = src_ip[11:8];
			29  : fifo_rddata = src_ip[15:12];
			30  : fifo_rddata = src_ip[3:0];
			31  : fifo_rddata = src_ip[7:4];
			
			//目的地址192.168.0.3
			32  : fifo_rddata = dst_ip[27:24];
			33  : fifo_rddata = dst_ip[31:28];
			34  : fifo_rddata = dst_ip[19:16];
			35  : fifo_rddata = dst_ip[23:20];
			36  : fifo_rddata = dst_ip[11:8];
			37  : fifo_rddata = dst_ip[15:12];
			38  : fifo_rddata = dst_ip[3:0];
			39  : fifo_rddata = dst_ip[7:4];
			
			//源端口号5000(0x1388)
			40  : fifo_rddata = src_port[11:8];
			41  : fifo_rddata = src_port[15:12];
			42  : fifo_rddata = src_port[3:0];
			43  : fifo_rddata = src_port[7:4];
			
			//目的端口号
			44  : fifo_rddata = dst_port[11:8];
			45  : fifo_rddata = dst_port[15:12];
			46  : fifo_rddata = dst_port[3:0];
			47  : fifo_rddata = dst_port[7:4];
			
			//UDP数据报总长度（UDP报头+数据）
			48  : fifo_rddata = udp_total_len[11:8];
			49  : fifo_rddata = udp_total_len[15:12];
			50  : fifo_rddata = udp_total_len[3:0];
			51  : fifo_rddata = udp_total_len[7:4];
			
			//UDP报头校验和  忽略
			52  : fifo_rddata = 4'h0;
			53  : fifo_rddata = 4'h0;
			54  : fifo_rddata = 4'h0;
			55  : fifo_rddata = 4'h0;
			
			//用户数据：
			default:fifo_rddata = rddata;
		endcase
	end

	rduw state_probe(
		.probe(state),
		.source()
	);
	
	rduw rduw(
		.probe(rdusedw),
		.source()
	);

	
	reg [15:0]delay_cnt;
	always@(posedge mii_tx_clk or negedge rst_n)
	if(!rst_n)begin
		state <= IDLE;
		tx_go <= 1'b0;
		delay_cnt <= 0;
	end
	else begin
		case(state)
			IDLE:				
				if(rdusedw >= {frame_bytes_len,1'b0})begin
					data_total_len <= frame_bytes_len;
					tx_go <= 1'b1;
					state <= START_TX;
				end
				else if(!packet_state_r[2])begin
					if(rdusedw >= 44)begin
						data_total_len <= rdusedw[12:1];
						tx_go <= 1'b1;
						state <= START_TX;
					end
				end
				else begin
					state <= IDLE;
					tx_go <= 1'b0;
					delay_cnt <= 0;
				end
				
			START_TX:
				begin
					tx_go <= 1'b0;
					state <= DELAY1;				
				end
	
			/*从使能发送到frame_done信号清零，有若干个时钟周期
			的延迟，因此必须等等待若干个时钟周期，待frame_done
			信号清零后再去等待frame_done信号重新被置1 */
			DELAY1: state <= DELAY2;
		
			DELAY2: state <= DELAY3;
			
			DELAY3: state <= WAITDONE;

			WAITDONE:
				if(send_done)begin
					state <= DELAY_BETWEEN_FRAME;
				end
				else
					state <= WAITDONE;
					
			DELAY_BETWEEN_FRAME:
				if(delay_cnt < 128)begin
					state <= DELAY_BETWEEN_FRAME;
					delay_cnt <= delay_cnt + 1'b1;
				end
				else begin
					state <= IDLE;
					delay_cnt <= 0;
				end
		
			default:state <= IDLE;
		endcase
	end
	
	//帧发送状态，为1表明发送完成，为1表明正在发送中
	always@(posedge mii_tx_clk or negedge rst_n)
	if(!rst_n)
		frame_done <= 1'b0;
	else if(send_done)
		frame_done <= 1'b1;
	else if(tx_go)
		frame_done <= 1'b0;
		
	//将帧发送状态信号从25Mhz时钟域同步到50MHz时钟域，作为50Mhz时钟域状态机跳转条件
	always@(posedge sink_dataclk)
		frame_done_r <= frame_done;

endmodule

