`timescale 1ns/1ns

module eth_send_test_tb;
	
	reg rst_n;	
	//MII 接口信号	
	reg mii_tx_clk;
	wire mii_tx_en;
	wire mii_tx_er;
	wire [3:0]mii_tx_data;
	wire phy_rst_n;
	
	reg  [11:0]frame_bytes_len;
	wire	[10:0]wrusedw;
	
	reg sink_datavalid;
	reg sink_dataclk;
	reg [15:0]sink_data;
//	reg sink_sop;
//	reg sink_eop;	
	
	wire sink_sop;
	wire sink_eop;	

	eth_send_test eth_send_test(
		.rst_n(rst_n),
		.sink_datavalid(sink_datavalid),
		.sink_data(sink_data),
		.sink_dataclk(sink_dataclk),
		.sink_sop(sink_sop),
		.sink_eop(sink_eop),
		
		.aclr(1'b0),
		.frame_bytes_len(frame_bytes_len),
		.wrusedw(wrusedw),
		.mii_tx_clk(mii_tx_clk),
		.mii_tx_en(mii_tx_en),
		.mii_tx_er(mii_tx_er),
		.mii_tx_data(mii_tx_data),
		.phy_rst_n(phy_rst_n),
		.mii_rx_clk(),
		.mii_rx_dv(),
		.mii_rx_er(),
		.mii_rx_data()
	);
	
	initial mii_tx_clk = 1;
	always #20 mii_tx_clk = ~mii_tx_clk;
	
	initial sink_dataclk = 1;
	always #10 sink_dataclk = ~sink_dataclk;
	
	initial begin
		rst_n = 0;
		frame_bytes_len = 640*2;
		sink_datavalid = 0;
		sink_data = 0;	
		sink_sop = 0;
		sink_eop = 0;
		#201;
		rst_n = 1;
		#200;
		#200000;
		sink_sop = 1;
		#20;
		sink_sop = 0;
		repeat(20)begin
			repeat(500)begin
				sink_datavalid = 1;
				sink_data <= sink_data + 1'b1;
				#20;			
			end
			sink_datavalid = 0;
			#200000;
		end
		sink_eop = 1;
		#20;
		sink_eop = 0;
		#2000;
		$stop;
	end
	
	reg clk_tft;
	
	initial clk_tft = 1;
	always #500 clk_tft = ~clk_tft;
	
	TFT_CTRL TFT_CTRL(
		.Clk9M(clk_tft),	//系统输入时钟9MHZ
		.Rst_n(1),
		.data_in(tft_data_in),	//待显示数据
		.hcount(),		//TFT行扫描计数器
		.vcount(),		//TFT场扫描计数器
		.TFT_RGB({TFT_RED[7:3], TFT_GREEN[7:2], TFT_BLUE[7:3]}),	//TFT数据输出
		.TFT_HS(TFT_HS),		//TFT行同步信号
		.TFT_VS(TFT_VS),		//TFT场同步信号
		.TFT_CLK(TFT_CLK),
		.TFT_DE(TFT_DE),
		.TFT_PWM(TFT_PWM)
	);
	
	reg en_eth_send_image;
	reg [1:0] TFT_VS_r;
	always@(posedge clk_tft)
		TFT_VS_r <= {TFT_VS_r[0], TFT_VS};
	
	assign sink_sop = (TFT_VS_r == 2'b01) & en_eth_send_image;
	assign sink_eop = (TFT_VS_r == 2'b10) & en_eth_send_image;
	
	always@(posedge clk_tft or negedge reset_n)
	if(!reset_n)
		en_eth_send_image <= 1'b0;
	else if(en_send)
		en_eth_send_image <= 1'b1;
	else if(sink_eop)
		en_eth_send_image <= 1'b0;

endmodule
