module udp_send_fifo_test(
	clk50m,
	rst_n,
	
	//MII 接口信号	
	mii_tx_clk,
	mii_tx_en,
	mii_tx_er,
	mii_tx_data,
	
	mii_rx_clk,
	mii_rx_dv,
	mii_rx_er,
	mii_rx_data,
	
	phy_rst_n

);

	input clk50m;
	input rst_n;
	
	//MII 接口信号	
	input mii_tx_clk;	//MII接口发送时钟，由PHY芯片产生，25MHz
	output mii_tx_en;	//MII接口发送数据使能信号，高电平有效
	output mii_tx_er;	//发送错误，用以破坏数据包发送
	output [3:0]mii_tx_data; //MII接口发送数据线，FPGA通过该数据线将需要发送的数据依次送给PHY芯片
	output phy_rst_n;	// PHY 复位信号

	input mii_rx_clk;	//MII接口接收时钟，由PHY芯片产生，25MHz
	input mii_rx_dv;	//MII接口接收数据有效信号，高电平有效
	input mii_rx_er;	//接收错误，本实例中暂时忽略该信号
	input [3:0]mii_rx_data;	//MII接口数据总线，FPGA通过该数据线读取PHY芯片接收到的以太网数据

	reg  [11:0]frame_bytes_len;
	wire	[10:0]wrusedw;
	
	reg sink_datavalid;
	wire sink_dataclk;
	reg [15:0]sink_data;
	wire en_write;
	assign sink_dataclk = clk50m;
	
	wire sink_sop;
	wire sink_eop;
	
	eth_send_test eth_send_test(
			.rst_n(rst_n),
			.sink_datavalid(sink_datavalid),
			.sink_data(sink_data),
			.sink_dataclk(sink_dataclk),
			.sink_sop(sink_sop),
			.sink_eop(sink_eop),
			
			.aclr(1'b0),
			.frame_bytes_len(1280),
			.wrusedw(wrusedw),
			.mii_tx_clk(mii_tx_clk),
			.mii_tx_en(mii_tx_en),
			.mii_tx_er(mii_tx_er),
			.mii_tx_data(mii_tx_data),
			.phy_rst_n(phy_rst_n),
			.mii_rx_clk(),
			.mii_rx_dv(),
			.mii_rx_er(),
			.mii_rx_data()
		);
	
	//	
	reg [1:0]state;
	always@(posedge clk50m or negedge rst_n)
	if(!rst_n)
		state <= 0;
	else if((wrusedw <= 1280) && en_write)
		state <= 1;
	else if((wrusedw >= 1920) | !en_write)
		state <= 0;
	else
		state <= state;
		
	always@(posedge clk50m or negedge rst_n)
	if(!rst_n)begin
		sink_datavalid = 0;
		sink_data <= 0;
	end	
	else if(state)begin
		sink_datavalid = 1;
		sink_data <= sink_data + 1'b1;	
	end
	else begin
		sink_datavalid = 0;
		sink_data <= sink_data;
	end
	
	reg [15:0]dly_cnt;
	always@(posedge clk50m or negedge rst_n)
	if(!rst_n)
		dly_cnt <= 16'd0;
	else
		dly_cnt <= dly_cnt + 1'b1;
	
	assign sink_sop = (dly_cnt == 1);
	assign sink_eop = (dly_cnt == 16'h7f00);
	
	assign en_write = (dly_cnt >= 1) & (dly_cnt <= 16'h7f00);
	
endmodule
