module OV7xxx_Init(
	Clk,
	Rst_n,
	Init_en,
	
	Coms_scl,
	Coms_sda,
	
	Init_done
);

	parameter OV7670CFG_ADDR   = 8'h00;
	parameter OV7725CFG_ADDR   = 8'h80;
	
	parameter OV7670CFG_MCNT   = 8'd115;
	parameter OV7725CFG_MCNT   = 8'd67;
	
	parameter GET_OV7xxx_RADDR = 16'h0a0b;
	parameter OV7670_TYPE_R    = 16'h7673;
	parameter OV7725_TYPE_R    = 16'h7721;

	input Clk;
	input Rst_n;
	input Init_en;
	
	output Coms_scl;
	inout Coms_sda;
	
	output reg Init_done;
	
	reg [7:0]Word_addr;
	reg Wr;
	reg [7:0]Wr_data;
	wire Wr_data_vaild;
	reg Rd;
	wire [7:0]Rd_data;
	wire Rd_data_vaild;
	
	wire Wr_done;
	wire Rd_done;
	
	reg [7:0]ov7xxxcfg_addr;
	wire [15:0]ov7xxxcfg_reg;
		
	reg sccb_done;
	
	reg [7:0]VER;
	reg [7:0]PID;
	
	reg [3:0]state;
	
	localparam IDLE     = 4'b0001,
	           GET_TYPE = 4'b0010,
				  PARA_CFG = 4'b0100,
				  DONE     = 4'b1000;	

	//例化SCCB控制器模块
	sccb sccb(
		.Clk(Clk),
		.Rst_n(Rst_n),
		
		.Device_addr(8'h42),
		.Word_addr(Word_addr),
		
		.Wr(Wr),
		.Wr_data(Wr_data),
		.Wr_data_vaild(Wr_data_vaild),
		.Rd(Rd),
		.Rd_data(Rd_data),
		.Rd_data_vaild(Rd_data_vaild),
		
		.Scl(Coms_scl),
		.Sda(Coms_sda),
		.Wr_done(Wr_done),
		.Rd_done(Rd_done)
	);

	//ov7xxx配置文件
	ov7xxxcfg ov7xxxcfg(
		.address(ov7xxxcfg_addr),
		.clock(Clk),
		.q(ov7xxxcfg_reg)
	);
	
	//读取摄像头寄存器个数计数器
	reg [1:0]rd_type_cnt;
	always@(posedge Clk or negedge Rst_n)
	begin
		if(!Rst_n)
			rd_type_cnt <= 2'd0;
		else if(state == GET_TYPE)begin
			if(Rd_done)
				rd_type_cnt <= rd_type_cnt + 2'd1;
			else
				rd_type_cnt <= rd_type_cnt;	
		end
		else
			rd_type_cnt <= 2'd0;		
	end
	
	//读取摄像头型号寄存器值
	reg [15:0]rd_type_reg;
	always@(posedge Clk or negedge Rst_n)
	begin
		if(!Rst_n)
			rd_type_reg <= 16'h0;
		else if(Rd_done && state == GET_TYPE)
			rd_type_reg <= {rd_type_reg[7:0],Rd_data};
		else
			rd_type_reg <= rd_type_reg;			
	end
	
	//配置摄像头寄存器个数计数器
	reg [7:0]wr_type_cnt;
	always@(posedge Clk or negedge Rst_n)
	begin
		if(!Rst_n)
			wr_type_cnt <= 8'd0;
		else if(state == PARA_CFG)begin
			if(Wr_done)
				wr_type_cnt <= wr_type_cnt + 8'd1;
			else
				wr_type_cnt <= wr_type_cnt;			
		end
		else
			wr_type_cnt <= 8'd0;			
	end
	
	//主状态机
	always@(posedge Clk or negedge Rst_n)
	begin
		if(!Rst_n)begin
			state <= IDLE;
			Word_addr <= 8'h00;
			Wr <= 1'b0;
			Wr_data <= 8'h00;
			Rd <= 1'b0;
			ov7xxxcfg_addr <= 8'd0;
			Init_done <= 1'b0;
		end
		else begin
			case(state)
				IDLE:begin
					Word_addr <= 8'h00;
					Wr <= 1'b0;
					Wr_data <= 8'h00;
					Rd <= 1'b0;
					ov7xxxcfg_addr <= 8'd0;
					Init_done <= 1'b0;
					if(Init_en)
						state <= GET_TYPE;
					else
						state <= IDLE;
				end
				
				GET_TYPE:begin
					case(rd_type_cnt)
						2'd0:begin
							Rd <= 1'b1;
							Word_addr <= GET_OV7xxx_RADDR[15:8];
							state <= GET_TYPE;
						end
						
						2'd1:begin
							Rd <= 1'b0;
							Word_addr <= GET_OV7xxx_RADDR[7:0];
							state <= GET_TYPE;
						end
						
						2'd2:begin
							state <= PARA_CFG;
							Rd <= 1'b0;
						end
						
						default:begin
							state <= IDLE;
							Rd <= 1'b0;
						end
					endcase				
				end
				
				PARA_CFG:begin
					if(rd_type_reg == OV7670_TYPE_R) begin

						ov7xxxcfg_addr <= wr_type_cnt + OV7670CFG_ADDR;
						
						if(wr_type_cnt == OV7670CFG_MCNT)begin
							state <= DONE;
							Wr <= 1'b0;
						end
						else begin
							state <= PARA_CFG;
							Word_addr <= ov7xxxcfg_reg[15:8];
							Wr_data <= ov7xxxcfg_reg[7:0];
							
							if(wr_type_cnt == OV7670CFG_MCNT-1)
								Wr <= 1'b0;
							else							
								Wr <= 1'b1;							
						end
					end					
					else if(rd_type_reg == OV7725_TYPE_R)begin
					
						ov7xxxcfg_addr <= wr_type_cnt + OV7725CFG_ADDR;
					
						if(wr_type_cnt == OV7725CFG_MCNT)begin
							state <= DONE;
							Wr <= 1'b0;
						end
						else begin
							state <= PARA_CFG;							
							Word_addr <= ov7xxxcfg_reg[15:8];
							Wr_data <= ov7xxxcfg_reg[7:0];
							
							if(wr_type_cnt == OV7725CFG_MCNT-1)
								Wr <= 1'b0;
							else
								Wr <= 1'b1;							
						end
					end
					else begin
						state <= IDLE;
						Wr <= 1'b0;
					end
				end
				
				DONE:begin
					state <= IDLE;
					Init_done <= 1'b1;									
				end
				
				default: state <= IDLE;				
			endcase
		end
	end

endmodule 